# Generated by Django 6.0.2 on 2026-02-16 15:29

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="ReportRequest",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=200)),
                (
                    "report_type",
                    models.CharField(
                        choices=[
                            ("daily_summary", "Daily Summary"),
                            ("weekly_digest", "Weekly Digest"),
                            ("monthly_analysis", "Monthly Analysis"),
                        ],
                        max_length=50,
                    ),
                ),
                (
                    "num_sections",
                    models.PositiveIntegerField(
                        default=3,
                        help_text="Number of sections to include in the report.",
                    ),
                ),
                (
                    "dagster_run_id",
                    models.CharField(blank=True, editable=False, max_length=64),
                ),
                (
                    "status",
                    models.CharField(
                        blank=True, default="", editable=False, max_length=32
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "created_by",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
            },
        ),
    ]
